@@ -15,10 +15,10 @@ Copyright 2017 Steven Diamond
 """
 from cvxpy.lin_ops import lin_op as lo
 import cvxpy.cvxcore.python.cvxcore as cvxcore
+import cvxpy.settings as s
 import numbers
 import numpy as np
 import scipy.sparse
-from collections import deque
 
 
 def get_parameter_vector(param_size,
@@ -107,17 +107,11 @@ def reduce_problem_data_tensor(A, var_length, quad_form=False):
     # construct a reduced COO matrix
     A.eliminate_zeros()
     A_coo = A.tocoo()
-    unique_old_row = np.sort(np.unique(A_coo.row))
-    old_row_to_new_row = {
-        unique_old_row[i]: i for i in range(unique_old_row.size)
-    }
+
+    unique_old_row, reduced_row = np.unique(A_coo.row, return_inverse=True)
     reduced_A_shape = (unique_old_row.size, A_coo.shape[1])
 
     # remap the rows
-    old_row = A_coo.row
-    reduced_row = np.zeros(old_row.shape)
-    for i in range(reduced_row.size):
-        reduced_row[i] = old_row_to_new_row[old_row[i]]
     reduced_A = scipy.sparse.coo_matrix(
         (A_coo.data, (reduced_row, A_coo.col)), shape=reduced_A_shape)
 
@@ -139,22 +133,10 @@ def reduce_problem_data_tensor(A, var_length, quad_form=False):
     # construction of the indptr: scan through cols, and find
     # the structure of the column index pointer
     indptr = np.zeros(n_cols + 1, dtype=np.int32)
-    i = 0
-    j = 1
-    nnz = cols.size
-    prev_col = 0
-    while i < nnz:
-        col = cols[i]
-        while col == prev_col and i < nnz - 1:
-            i += 1
-            col = cols[i]
-        if prev_col != col:
-            for k in range(col - prev_col):
-                indptr[j + k] = i
-            j += col - prev_col
-        prev_col = col
-        i += 1
-    indptr[j:] = nnz
+    positions, counts = np.unique(cols, return_counts=True)
+    indptr[positions+1] = counts
+    indptr = np.cumsum(indptr)
+
     return reduced_A, indices, indptr, shape
 
 
@@ -327,10 +309,12 @@ def get_problem_matrix(linOps,
         build_lin_op_tree(lin, linPy_to_linC)
         tree = linPy_to_linC[lin]
         lin_vec.push_back(tree)
+
     problemData = cvxcore.build_matrix(lin_vec,
                                        int(var_length),
                                        id_to_col_C,
-                                       param_to_size_C)
+                                       param_to_size_C,
+                                       s.get_num_threads())
 
     # Populate tensors with info from problemData.
     tensor_V = {}
@@ -367,7 +351,9 @@ def get_problem_matrix(linOps,
     I = np.concatenate(I)
     J = np.concatenate(J)
     A = scipy.sparse.csc_matrix(
-        (V, (I, J)), shape=(np.int64(constr_length)*np.int64(var_length+1), param_size_plus_one))
+        (V, (I, J)),
+        shape=(np.int64(constr_length)*np.int64(var_length+1),
+               param_size_plus_one))
     return A
 
 
@@ -425,7 +411,7 @@ def get_type(linPy):
         raise NotImplementedError("Type %s is not supported." % ty)
 
 
-def set_matrix_data(linC, linPy):
+def set_matrix_data(linC, linPy) -> None:
     """Calls the appropriate cvxcore function to set the matrix data field of
        our C++ linOp.
     """
@@ -439,7 +425,7 @@ def set_matrix_data(linC, linPy):
         linC.set_data_ndim(len(linPy.data.shape))
 
 
-def set_slice_data(linC, linPy):
+def set_slice_data(linC, linPy) -> None:
     """
     Loads the slice data, start, stop, and step into our C++ linOp.
     The semantics of the slice operator is treated exactly the same as in
@@ -453,7 +439,7 @@ def set_slice_data(linC, linPy):
         linC.push_back_slice_vec(slice_vec)
 
 
-def set_linC_data(linC, linPy):
+def set_linC_data(linC, linPy) -> None:
     """Sets numerical data fields in linC."""
     assert linPy.data is not None
     if isinstance(linPy.data, tuple) and isinstance(linPy.data[0], slice):
@@ -466,7 +452,7 @@ def set_linC_data(linC, linPy):
         set_matrix_data(linC, linPy)
 
 
-def make_linC_from_linPy(linPy, linPy_to_linC):
+def make_linC_from_linPy(linPy, linPy_to_linC) -> None:
     """Construct a C++ LinOp corresponding to LinPy.
 
     Children of linPy are retrieved from linPy_to_linC.
@@ -492,7 +478,7 @@ def make_linC_from_linPy(linPy, linPy_to_linC):
             set_linC_data(linC, linPy)
 
 
-def build_lin_op_tree(root_linPy, linPy_to_linC):
+def build_lin_op_tree(root_linPy, linPy_to_linC) -> None:
     """Construct C++ LinOp tree from Python LinOp tree.
 
     Constructed C++ linOps are stored in the linPy_to_linC dict,
