@@ -1,28 +1,25 @@
 """
 Copyright 2013 Steven Diamond
 
-This file is part of CVXPY.
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-CVXPY is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+    http://www.apache.org/licenses/LICENSE-2.0
 
-CVXPY is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with CVXPY.  If not, see <http://www.gnu.org/licenses/>.
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 """
 
 import cvxpy.interface.matrix_utilities
 import abc
-import numbers
 import numpy as np
 
-class BaseMatrixInterface(object):
+
+class BaseMatrixInterface:
     """
     An interface between constants' internal values
     and the target matrix used internally.
@@ -40,7 +37,7 @@ class BaseMatrixInterface(object):
         Returns:
             A matrix of type self.target_matrix or a scalar.
         """
-        return NotImplemented
+        raise NotImplementedError()
 
     # Adds a case for scalars to const_to_matrix methods.
     @staticmethod
@@ -55,47 +52,51 @@ class BaseMatrixInterface(object):
     # Return an identity matrix.
     @abc.abstractmethod
     def identity(self, size):
-        return NotImplemented
+        raise NotImplementedError()
+
+    # Return the number of elements of the matrix.
+    def size(self, matrix):
+        return np.prod(self.shape(matrix), dtype=int)
 
     # Return the dimensions of the matrix.
     @abc.abstractmethod
-    def size(self, matrix):
-        return NotImplemented
+    def shape(self, matrix):
+        raise NotImplementedError()
 
     # Get the matrix interpreted as a scalar.
     @abc.abstractmethod
     def scalar_value(self, matrix):
-        return NotImplemented
+        raise NotImplementedError()
 
     # Return a matrix with all 0's.
-    def zeros(self, rows, cols):
-        return self.scalar_matrix(0, rows, cols)
+    def zeros(self, shape):
+        return self.scalar_matrix(0, shape)
 
     # Return a matrix with all 1's.
-    def ones(self, rows, cols):
-        return self.scalar_matrix(1, rows, cols)
+    def ones(self, shape):
+        return self.scalar_matrix(1, shape)
 
     # A matrix with all entries equal to the given scalar value.
     @abc.abstractmethod
-    def scalar_matrix(self, value, rows, cols):
-        return NotImplemented
+    def scalar_matrix(self, value, shape):
+        raise NotImplementedError()
 
     # Return the value at the given index in the matrix.
     def index(self, matrix, key):
         value = matrix[key]
         # Reduce to a scalar if possible.
-        if cvxpy.interface.matrix_utilities.size(value) == (1,1):
+        if cvxpy.interface.matrix_utilities.shape(value) == (1, 1):
             return cvxpy.interface.matrix_utilities.scalar_value(value)
         else:
             return value
 
     # Coerce the matrix into the given shape.
     @abc.abstractmethod
-    def reshape(self, matrix, size):
-        return NotImplemented
+    def reshape(self, matrix, shape):
+        raise NotImplementedError()
 
     def block_add(self, matrix, block, vert_offset, horiz_offset, rows, cols,
-                  vert_step=1, horiz_step=1):
+                  vert_step: int = 1, horiz_step: int = 1) -> None:
         """Add the block to a slice of the matrix.
 
         Args:
@@ -123,7 +124,8 @@ class BaseMatrixInterface(object):
         """
         # If the block is a scalar, promote it.
         if cvxpy.interface.matrix_utilities.is_scalar(block):
-            block = self.scalar_matrix(cvxpy.interface.matrix_utilities.scalar_value(block), rows, cols)
+            block = self.scalar_matrix(
+                cvxpy.interface.matrix_utilities.scalar_value(block), rows, cols)
         # If the block is a vector coerced into a matrix, promote it.
         elif cvxpy.interface.matrix_utilities.is_vector(block) and cols > 1:
             block = self.reshape(block, (rows, cols))
